require 'uu/os/qos/qos_handler'
require 'uu/os/rest'
require 'uu/os/uesuri'
require 'uu/os/access_role/access_role_attributes'
require 'uu/os/access_role/access_role_set_attributes'
require 'uu/os/access_role/access_role_create'
require 'uu/os/access_role/access_role_get_role_interface_list'
require 'uu/os/access_role/access_role_remove_role_interface'
require 'uu/os/access_role/access_role_add_role_interface'
require 'uu/os/access_role/access_role_get_role_interface_list_item'

module UU
  module OS
    # Access Role service.
    # @deprecated Use {UU::OS::PersonalAccessRole} instead.
    module AccessRole
      extend self

      # Service path of the REST API.
      PATH = 'ues/core/role/UESAccessRole'

      # This command returns attributes representing the personal access role specified
      # by the +access_role_uri+ parameter. The command does not change the state of the role
      # in the system.
      #
      # @param [String, UU::OS::UESURI] access_role_uri UESURI of a personal access role whose attributes are returned
      #
      # @return [AccessRoleAttributes] DTO containing the personal access role attributes
      #
      # @example
      #   # Get attributes of access role.
      #   getAttributes = UU::OS::AccessRole.get_attributes('ues:UNI:ACCESSROLE')
      #
      def get_attributes(access_role_uri)
        svc = UU::OS::REST::RemoteClient.new(AccessRole)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getAttributes', access_role_uri)
          return AccessRole::AccessRoleAttributes.new(res)
        end
      end

      # Command for setting attributes of an access role. The command can't modify references
      # to objects like location, competent role and so on.
      #
      # @param [String, UU::OS::UESURI] access_role_uri UESURI of the access role whose attributes are to be changed
      # @param [AccessRoleSetAttributes] access_role DTO containing new attributes theaccess role
      # @option access_role [String] :first_name The new first name of the access role. When nil is set, the first name is not modified.
      # @option access_role [String] :surname The new surname of the access role. When nil is set, the surname is not modified.
      # @option access_role [String] :code The new code of the access role. When nil is set, the code is not modified. When blank string is set, the code is generated automatically using the system sequence. The code must be unique in a given territory, otherwise the setAttributes command fails.
      # @option access_role [String] :description The new description of the access role. When nil is set, the description is not modified.
      # @option access_role [UU::OS::Artifact::SecurityLevel] :security_level Security level of the new access role. When nil is set, security level is taken from the meta artifact SecurityLevel. If the meta artifact SecurityLevel is higher than the user's security clearance, the create command fails.
      # @option access_role [UU::OS::Artifact::SecurityLevel] :security_clearance Security clearance of the new access role. When nil is set, the security clearance is not modified.
      # @option access_role [Boolean] :searchable_in_territory Specifies if it is possible to search the access role in its territory. When nil is set, this attribute on the access role is not modified.
      # @option access_role [String, UU::OS::UESURI] :icon_uri URI of the role's icon. When nil is set, the icon is not modified. If nil UESURI ("ues:{[-1]}:{[-1]}:")  is set, the role icon is removed and the role uses icon from meta artifact. Allowed format of icon URI: 'ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}CODE_OF_ICON}'. An example of icon URI: 'ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}ART_077}'.
      # @option access_role [Boolean] :grant_all_from_organizational_unit <b>Deprecated.</b> Specifies if the access role will be included among All roles from organizational unit. When nil is set, this attribute on the access role is not modified.
      # @option access_role [Boolean] :component_content Defines whether the primarily displayed content of the created Access role is component content. Default value is false.           
      #
      # @return [UU::OS::UESURI] UESURI of the access role
      #
      # @example
      #   # Set access role confidential and not search able in territory.
      #   accessRoleUri = UU::OS::AccessRole.set_attributes('ues:UNI:ACCESSROLE', :first_name=>'John', :surname=>'Doe', :code=>'NEWACCESSCODE', :description=>'Some basic access role',:security_level=>UU::OS::Artifact::SecurityLevel::CONFIDENTIAL,:searchable_in_territory=>false)
      #
      def set_attributes(access_role_uri, access_role = nil)
        svc = UU::OS::REST::RemoteClient.new(AccessRole)
        payload = UU::OS::AccessRole::AccessRoleSetAttributes.new(access_role).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setAttributes', access_role_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Creates a new access role. The command creates a new access role in aspecified location.
      # At the very least meta artifact must bespecified in {AccessRoleCreate}
      # DTO. Competent role for the new role will be selected as the most suitable according
      # to the specified container and meta artifact (executive/authorized role with
      # connected interface), or can be also specified in {AccessRoleCreate}
      # DTO.
      #
      # @param [String, UU::OS::UESURI] location_uri UESURI of the folder/org. unit/meta model where the access role will be created.
      # @param [AccessRoleCreate] access_role DTO containing attributes of the new access role
      # @option access_role [String, UU::OS::UESURI] :meta_artifact_uri UESURI of the meta artifact. Only a meta artifact with the template class of an Access role (not its descendants) can be used. If the rights does not permit to create a new access role from the meta artifact in the specified location, the create command fails.
      # @option access_role [String] :universe_id Universe identifier of the user. When nil is set, the create command fails.
      # @option access_role [String] :first_name First name of the new access role. When nil is set, the create command fails.
      # @option access_role [String] :surname Surname of the new access role. When nil is set, the create command fails.
      # @option access_role [String] :code Code of the new access role. When nil is set, the code is generated automatically using the system sequence. The code must be unique in the given territory, otherwise the create command fails.
      # @option access_role [String] :description Description of the new access role. When nil is set, description from meta artifact is used if set, otherwise no description is set on the new access role.
      # @option access_role [String, UU::OS::UESURI] :competent_role_uri  UESURI of the competent role. When nil is set, the executive role from authorization is used. If no role is authorized to create the access role into specified container, the create command fails. If the selected role doesn't have connected the access role interface with the selected meta artifact, the create command fails. If the selected role doesn't belong to the same organizational unit as anew role or if it doesn't belong to one of superior organizational units, the create command fails.
      # @option access_role [UU::OS::Artifact::SecurityLevel] :security_level Security level. When nil is set, security level is taken from the meta artifact default security level. If the meta artifact template security level is higher than the user's security clearance, the create command fails.
      # @option access_role [UU::OS::Artifact::SecurityLevel] :security_clearance  Security clearance of the new access role. When nil is set, the value {UU::OS::Artifact::SecurityLevel::NO_CONFIDENTIALITY NO_CONFIDENTIALITY} is used.
      # @option access_role [Boolean] :searchable_in_territory Specifies if it is possible to search the new access role in its territory. When nil is set, SearchableInTerritory from meta artifact is used if set, otherwise the create command fails.
      # @option access_role [String, UU::OS::UESURI] :icon_uri URI of the role's icon. When nil is set, the icon is not modified. If nil UESURI ("ues:{[-1]}:{[-1]}:")  is set, the role icon is removed and the role uses icon from meta artifact. Allowed format of icon URI: 'ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}CODE_OF_ICON}'. An example of icon URI: 'ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}ART_077}'.
      # @option access_role [Boolean] :grant_all_from_organizational_unit <b>Deprecated.</b> Specifies if the access role will be included among All roles from organizational unit. When true or nil is set, the access role will be included, otherwise it will not be included among All roles from organizational unit.
      # @option access_role [Boolean] :component_content Defines whether the primarily displayed content of the created Access role is component content. Default value is false.           
      #
      # @return [UU::OS::UESURI] UESURI of the new access role 
      # @raise [UESAccessRoleRTException]
      #
      # @example
      #   # Create access role in specified location.
      #   UU::OS::AccessRole.create('ues:UNI:FOLDER', :meta_artifact_uri=>'ues:TER:METAARTIFACT',:universe_id=>'37820001', :first_name=>'John', :surname=>'Doe', :code=>'DOECODE', :description=>'Some sample description',:searchable_in_territory=>true)
      #
      def create(location_uri, access_role = nil)
        svc = UU::OS::REST::RemoteClient.new(AccessRole)
        payload = UU::OS::AccessRole::AccessRoleCreate.new(access_role).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('create', location_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Returns list of the role interfaces that are connected to the specified access
      # role. The list of returned interfaces could be filtered by the name or code. Returned
      # list does not contain any objects filtered out by executed UESQuery. The list is
      # sorted by the name (and by code in case are equal) of an interface by default.
      #
      # @param [String, UU::OS::UESURI] access_role_uri UESURI of the access role for which the connected role interfaces are to be listed
      # @param [AccessRoleGetRoleInterfaceList] criteria DTO containing criteria for listing access role interfaces. Available parameters are: {AccessRoleGetRoleInterfaceListItem#name name} and {AccessRoleGetRoleInterfaceListItem#code code}.See {https://uu.unicornuniverse.eu/ues/sesm?&SessFree=ues:UNI-BT:UES.DOC%2FCMD_V1%2FUESQUERY UESQuery Documentation}.
      # @option criteria [String] query # The UES query string. It has to conform the uesquery syntax (see documentation for more info). Filtering and order of the result list can be specified there. When nil is set, no filtering is applied and entries are ordered by name of casted subjects (and code as the second criteria). See {https://uu.unicornuniverse.eu/ues/sesm?&SessFree=ues:UNI-BT:UES.DOC%2FCMD_V1%2FUESQUERY UESQuery Documentation}.
      #
      # @return [UU::OS::REST::ResultList<AccessRole::AccessRoleGetRoleInterfaceList, AccessRole::AccessRoleGetRoleInterfaceListItem>] List of DTO representing role interfaces which are connected to the specified access role
      #
      # @example
      #   # Get role interface list for specified access role.
      #   roleIfcList = UU::OS::AccessRole.get_role_interface_list('ues:TERRITORY:ACCESSROLE')
      #
      def get_role_interface_list(access_role_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(AccessRole)
        dto = AccessRole::AccessRoleGetRoleInterfaceList.new(criteria)
        svc.add_parameter('pageIndex', dto.page_index)
        svc.add_parameter('pageSize', dto.page_size)
        svc.add_parameter('query', dto.query)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getRoleInterfaceList', access_role_uri)
          return UU::OS::REST::ResultList.new(AccessRole::AccessRoleGetRoleInterfaceList, AccessRole::AccessRoleGetRoleInterfaceListItem, res)
        end
      end

      # The command removes role interface connection from the specified access role.
      # Only connected interfaces can be disconnected from the access role.
      #
      # @param [String, UU::OS::UESURI] access_role_uri UESURI of the access role where the role interface will be disconnected
      # @param [AccessRoleRemoveRoleInterface] access_role DTO containing role interface that to be disconnected
      # @option access_role [String, UU::OS::UESURI] :role_interface_uri UESURI of the role interface that to be unconnected from the specified access role. When nil is set, the command fails.
      #
      # @example
      #   # Remove role interface from specified access role.
      #   UU::OS::AccessRole.remove_role_interface('ues:TERRITORY:ACCESSROLE', :role_interface_uri=>'ues:TERRITORY:ROLEIFC')
      #
      def remove_role_interface(access_role_uri, access_role = nil)
        svc = UU::OS::REST::RemoteClient.new(AccessRole)
        payload = UU::OS::AccessRole::AccessRoleRemoveRoleInterface.new(access_role).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('removeRoleInterface', access_role_uri, payload)
        end
      end

      # This command creates new role interface connection to the specified access role.
      # The role interface has to be in an active state and mustn't be already connected.
      # The role interface has to be connected to the one of the authorized roles for this
      # command, or the authorized role has to be casted to the default group of meta model where
      # the role interface is located.
      #
      # @param [String, UU::OS::UESURI] access_role_uri UESURI of the access role where the role interface will be connected
      # @param [AccessRoleAddRoleInterface] access_role DTO containing role interface that to be connected
      # @option access_role [String, UU::OS::UESURI] :role_interface_uri UESURI of the role interface that to be connected to the specified access role. When nil is set, the command fails.
      #
      # @example
      #   # Add role interface for specified access role.
      #   UU::OS::AccessRole.add_role_interface('ues:TERRITORY:ACCESSROLE', :role_interface_uri=>'ues:TERRITORY:ROLEIFC')
      #
      def add_role_interface(access_role_uri, access_role = nil)
        svc = UU::OS::REST::RemoteClient.new(AccessRole)
        payload = UU::OS::AccessRole::AccessRoleAddRoleInterface.new(access_role).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('addRoleInterface', access_role_uri, payload)
        end
      end

      # This command deactivates an access role. The access role will lose access to its
      # territory.
      #
      # @param [String, UU::OS::UESURI] access_role_uri UESURI of an access role that the access will be canceled.
      #
      # @example
      #   # Deactivate access for specified access role.
      #   UU::OS::AccessRole.deactivate_access('ues:TERRITORY:ACCESSROLE')
      #
      def deactivate_access(access_role_uri)
        svc = UU::OS::REST::RemoteClient.new(AccessRole)
        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('deactivateAccess', access_role_uri)
        end
      end

      # This command activates an access role. The access role will have access to its territory.
      #
      # @param [String, UU::OS::UESURI] access_role_uri UESURI of an access role that the access will be reactivated.
      # 
      # @example
      #   # Activate access for specified access role.
      #   UU::OS::AccessRole.activate_access('ues:TERRITORY:ACCESS_ROLE')
      #
      def activate_access(access_role_uri)
        svc = UU::OS::REST::RemoteClient.new(AccessRole)
        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('activateAccess', access_role_uri)
        end
      end

    end
  end

end
