require 'uu/os/qos/qos_handler'
require 'uu/os/rest'
require 'uu/os/uesuri'
require 'uu/os/activity_template/activity_template_attributes'
require 'uu/os/activity_template/activity_template_create'
require 'uu/os/activity_template/activity_template_get_activity_template_list'
require 'uu/os/activity_template/activity_template_set_attributes'

# Module Unicorn Universe.
module UU
  # Module Operating System.
  module OS
    # Module Activity Template.
    module ActivityTemplate
      extend self

      # Service path
      PATH = 'uu/os/ActivityTemplate'
      # Creates a new activity template either on the specified meta artifact or on the specified activity template
      # (in the case of creating a new template for subactivities). At least, its future name and type have to be specified in this command.
      #
      # @param [String, UU::OS::UESURI] parent_node_uri <b>Not nil.</b> URI of the meta artifact (or activity template) the new activity template should be created on.
      # @param [UU::OS::ActivityTemplate::ActivityTemplateCreate] options Specific options.
      # @option options [String] :name <b>Not nil.</b> Name of the new activity template.
      # @option options [String] :code Code of the new activity template. If not specified, the code is generated automatically using the system sequence. The code must be unique in the scope of the given meta artifact, otherwise the create command fails.
      # @option options [String] :description Description of the new activity template. If not specified, it is left empty.
      # @option options [String, UU::OS::Activity::ActivityType] :type <b>Not nil.</b> Type of the new activity template. Specific generic activity template will be used as the source for the new activity template according to the specified type. Subactivity templates whose parent has the type TIME_RESERVATION must also have the type TIME_RESERVATION. All possible types can be found as constants in {UU::OS::Activity::ActivityType} class.
      # @option options [String, UU::OS::UESURI] :related_object_uri URI of the artifact or object which is somehow related to the activity template.
      # @option options [String, UU::OS::UESURI] :icon_uri URI of the new activity template's icon. The same icon will be used for created activity instances. If not specified, there will be used a default icon derived from the specified state type. Allowed format of icon: 'ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}CODE_OF_ICON}'.
      # @option options [Boolean] :allow_creation_through_ui Parameter specifies whether activities can be created according to this activity template through the user interface or not. The default value is true.
      # @option options [String] :notification Parameter specifies the default time from creation of the activity to its expected completion. The entered value must be in ISO 8601 format. The default value is one day. If the user creates a subactivity template, the default value is derived from the parent activity template. The value must not be longer than the value of the same parameter at the superior activity (if it exists).
      # @option options [String] :duration Parameter specifies the duration of the activity template. It can be set only for activity templates with type COVER_MEETING or TIME_RESERVATION. For those types of templates, the default value is one hour. The entered value must be in ISO 8601 format. If the user creates a subactivity template, the default value is derived from the parent activity template. The value must not be longer than the value of the same parameter at the superior activity (if it exists).
      # @option options [String, Boolean] :expiration Parameter specifies how long after the expected completion date the activities created according to this template should be automatically terminated. This parameter can contain two possible types of values: If it's a time duration, it sets the activity to expire after that duration. This value must be in ISO 8601 format. If it's set to false, the activity created according to this activity template will never expire. The default value is false. If the user creates a subactivity template, the default value is derived from the parent activity template. The value must not be longer than the value of the same parameter at the superior activity (if it exists).
      # @option options [Boolean] :allow_as_subactivity_template Parameter specifies whether this activity template will be usable as a template for subactivities. The default value is false.
      # @option options [String, UU::OS::UESURI] :default_subactivity_template_uri URI of the activity template which should be used as the default template for subactivities. The URI must refer to an activity template on the same meta artifact which is usable as delegation template. If it is not set, no default subactivity is specified. This parameter must not be set when the user creates a template for subactivities!
      # @option options [String, UU::OS::UESURI] :executive_role_uri <b>Not nil for subactivity templates.</b> URI of the executive role for the activities created according to this template.
      # @option options [Boolean] :block_time_in_calendar Specifies whether the activities created according to this template will reserve time in the related users' calendars. This parameter can be used only for activity templates with type COVER_MEETING or TIME_RESERVATION. For those types of templates, the default value is true.
      # @option options [Boolean] :allow_executive_role_change Parameter specifies whether Specifies whether the executive role of the activities created according to this template can be changed. The default value is true.
      # @option options [Boolean] :allow_subactivities Parameter specifies whether it is allowed to create subactivities of the activities created according to this template. The default value is true.
      #
      # @return [UU::OS::UESURI] URI of the new activity template.
      #
      # @example
      #   # Create an activity template with type DECIDE
      #   UU::OS::ActivityTemplate.create('ues:TERRITORY:META_ARTIFACT',
      #     :name => 'Activity template name',
      #     :code => 'Activity template code',
      #     :type => UU::OS::Activity::ActivityType::DECIDE,
      #     :description => 'Activity template description',
      #     :related_object_uri => 'ues:TERRITORY:ARTIFACT:OBJECT',
      #     :icon_uri => 'ues:#{system}:#{ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{ART_221}',
      #     :allow_creation_through_ui => true,
      #     :notification => 'P0DT1H0M0.000S',
      #     :expiration => 'P14DT0H0M0.000S',
      #     :allow_as_subactivity_template => true,
      #     :default_subactivity_template_uri => 'ues:TERRITORY:META_ARTIFACT:GENERIC_ACTIVITY_TEMPLATE',
      #     :executive_role_uri => 'ues:TERRITORY:ROLE',
      #     :allow_executive_role_change => true,
      #     :allow_subactivities => true)
      #
      #   # Create a template with type COVER_MEETING
      #   UU::OS::ActivityTemplate.create('ues:TERRITORY:META_ARTIFACT',
      #      :name => 'Activity template name',
      #      :code => 'Activity template code',
      #      :type => 'COVER_MEETING',
      #      :description => 'Activity template description',
      #      :related_object_uri => 'ues:TERRITORY:ARTIFACT:OBJECT',
      #      :icon_uri => 'ues:#{system}:#{ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{ART_221}',
      #      :allow_creation_through_ui => true,
      #      :notification => 'P1DT0H0M0.000S',
      #      :duration => 'P0DT1H0M0.000S',
      #      :expiration => false,
      #      :allow_as_subactivity_template => true,
      #      :default_subactivity_template_uri => 'ues:TERRITORY:META_ARTIFACT:GENERIC_ACTIVITY_TEMPLATE',
      #      :block_time_in_calendar => true,
      #      :executive_role_uri => 'ues:TERRITORY:ROLE',
      #      :allow_executive_role_change => true,
      #      :allow_subactivities => true)
      #
      #   # Create a template of a subactivity
      #   UU::OS::ActivityTemplate.create('ues:TERRITORY:META_ARTIFACT:ACTIVITY_TEMPLATE',
      #      :name => 'Activity template name',
      #      :code => 'Activity template code',
      #      :type => 'DECIDE',
      #      :description => 'Activity template description',
      #      :related_object_uri => 'ues:TERRITORY:ARTIFACT:OBJECT',
      #      :icon_uri => 'ues:#{system}:#{ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{ART_221}',
      #      :notification => 'P0DT1H0M0.000S',
      #      :expiration => 'P14DT0H0M0.000S',
      #      :allow_as_subactivity_template => true,
      #      :executive_role_uri => 'ues:TERRITORY:ROLE',
      #      :allow_executive_role_change => true,
      #      :allow_subactivities => true)
      def create(parent_node_uri, options)
        svc = UU::OS::REST::RemoteClient.new(ActivityTemplate)
        payload = UU::OS::ActivityTemplate::ActivityTemplateCreate.new(options).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('create', parent_node_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Returns attributes of the specified activity template.
      #
      # @param [String, UU::OS::UESURI] activity_template_uri <b>Not nil.</b> URI of the activity template whose attributes are to be returned.
      #
      # @return [UU::OS::ActivityTemplate::ActivityTemplateAttributes] Object with attributes of the activity template.
      #
      # @example
      #   # Get attributes of an activity template
      #   UU::OS::ActivityTemplate.get_attributes('ues:TERRITORY:META_ARTIFACT:ACTIVITY_TEMPLATE')
      def get_attributes(activity_template_uri)
        svc = UU::OS::REST::RemoteClient.new(ActivityTemplate)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getAttributes', activity_template_uri)
          return ActivityTemplate::ActivityTemplateAttributes.new(res)
        end
      end

      # Sets basic attributes of the specified activity template. This command cannot modify the type.
      #
      # @param [String, UU::OS::UESURI] activity_template_uri <b>Not nil.</b> URI of the activity template whose attributes are to be modified.
      # @param [UU::OS::ActivityTemplate::ActivityTemplateSetAttributes] options Specific options.
      # @option options [String] :name The new name of the activity template. If it is not specified, the name is not modified.
      # @option options [String] :code The new code of the activity template. If it is not specified, the code is not modified. If blank string is set, the code is generated automatically using the system sequence. The code must be unique in the scope of the given meta artifact, otherwise the setAttributes command fails.
      # @option options [String] :description The new description of the activity template. If it is not specified, the description is not modified.
      # @option options [String, UU::OS::UESURI] :related_object_uri The new related object of the activity template. If it is not specified, the related object reference is not modified. If nil URI ('ues:{[-1]}:{[-1]}:') is set, the related artifact is set empty.
      # @option options [String, UU::OS::UESURI] :icon_uri URI of the new activity template's icon. The same icon will be used for created activity instances. If it is not specified, the icon is not modified. If nil URI ('ues:{[-1]}:{[-1]}:') is set, the activity template's icon is replaced by a default icon derived from the type. Allowed format of icon: 'ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}CODE_OF_ICON}'.
      # @option options [Boolean] :allow_creation_through_ui Parameter specifies whether activities can be created according to this activity template through the user interface or not. This option cannot be turned off for generic templates. If it is not specified, the value is not modified.
      # @option options [String] :notification Parameter specifies the default time from creation of the activity to its expected completion. The entered value must be in ISO 8601 format. If it is not specified, the value is not modified. The value must not be longer than the value of the same parameter at the superior activity (if it exists).
      # @option options [String] :duration Parameter specifies the duration of the activity template. This parameter can be used only for activity templates with type COVER_MEETING or TIME_RESERVATION. The entered value must be in ISO 8601 format. If it is not specified, the value is not modified. The value must not be longer than the value of the same parameter at the superior activity (if it exists).
      # @option options [String, Boolean] :expiration Parameter specifies how long after the expected completion date the activities created according to this template should be automatically terminated. This parameter can contain two possible types of values: If it's a time duration, it sets the activity to expire after that duration. This value must be in ISO 8601 format. If it's set to false, the activity created according to this activity template will never expire. If it is not specified, the value is not modified. The value must not be longer than the value of the same parameter at the superior activity (if it exists).
      # @option options [Boolean] :allow_as_subactivity_template Parameter specifies whether this activity template will be usable as a template for subactivities. If it is not specified, the value is not modified.
      # @option options [String, UU::OS::UESURI] :default_subactivity_template_uri URI of the activity template which should be used as the default template for subactivities. The URI must refer to an activity template on the same meta artifact which is usable as delegation template. If it is not specified, the value is not modified. If nil URI ('ues:{[-1]}:{[-1]}:') is set, this parameter is set empty. This parameter must not be set on templates for subactivities!
      # @option options [String, UU::OS::UESURI] :executive_role_uri URI of the executive role for the activities created according to this template. If it is not specified, the value is not modified.If nil URI ('ues:{[-1]}:{[-1]}:') is set, the activity template's executive role is replaced by a default executive role which is the competent role of meta artifact in which activity template is located. 
      # @option options [Boolean] :block_time_in_calendar Specifies whether the activities created according to this template will reserve time in the related users' calendars. This parameter can be used only for activity templates with type COVER_MEETING or TIME_RESERVATION. If it is not specified, the value is not modified.
      # @option options [Boolean] :allow_executive_role_change Parameter specifies whether the executive role of the activities created according to this template can be changed. If it is not specified, the value is not modified.
      # @option options [Boolean] :allow_subactivities Parameter specifies whether it is allowed to create subactivities of the activities created according to this template. If it is not specified, the value is not modified.
      #
      # @return [UU::OS::UESURI] URI of the modified activity template.
      #
      # @example
      #   # Sets attributes of an activity template
      #   UU::OS::ActivityTemplate.set_attributes('ues:TERRITORY:META_ARTIFACT:ACTIVITY_TEMPLATE',
      #      :name => 'Activity template name',
      #      :code => 'Activity template code',
      #      :description => 'Activity template description',
      #      :related_object_uri => 'ues:TERRITORY:ARTIFACT:OBJECT',
      #      :icon_uri => 'ues:#{system}:#{ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{ART_221}',
      #      :allow_creation_through_ui => true,
      #      :notification => 'P0DT1H0M0.000S',
      #      :expiration => 'PT256H0M0.000S',
      #      :allow_as_subactivity_template => true,
      #      :allow_subactivities => true,
      #      :default_subactivity_template_uri => 'ues:TERRITORY:META_ARTIFACT:GENERIC_ACTIVITY_TEMPLATE',
      #      :executive_role_uri => 'ues:TERRITORY:ROLE',
      #      :allow_executive_role_change => true)
      def set_attributes(activity_template_uri, options)
        svc = UU::OS::REST::RemoteClient.new(ActivityTemplate)
        payload = UU::OS::ActivityTemplate::ActivityTemplateSetAttributes.new(options).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setAttributes', activity_template_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Deletes the activity template. It cannot be deleted in case there is an action or condition that contains
      # this template or if the template has already been used for creating another template. Also, it is not possible
      # to delete generic templates.
      #
      # @param [String, UU::OS::UESURI] activity_template_uri <b>Not nil.</b> URI of the activity template which is to be deleted.
      #
      # @example
      #   # Deletes an activity template
      #   UU::OS::ActivityTemplate.delete('ues:TERRITORY:META_ARTIFACT:ACTIVITY_TEMPLATE')
      def delete(activity_template_uri)
        svc = UU::OS::REST::RemoteClient.new(ActivityTemplate)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('delete', activity_template_uri)
        end
      end

      # Returns a list of activity templates of the specified meta artifact according to the entered criteria.
      #
      # @param [String, UU::OS::UESURI] meta_artifact_uri <b>Not nil.</b> URI of the meta artifact whose activity templates are to be listed.
      # @param [UU::OS::ActivityTemplate::ActivityTemplateGetActivityTemplateList] criteria Specific options.
      # @option criteria [String] :query Query for filtering and ordering of the result list. It has to conform the ues query syntax (see the documentation below for more info). If it is not specified, no filtering is applied and entries are ordered by the parameter name and code as the second criteria. Otherwise, it is possible to filter and sort the result list by the following attributes: name, code, type, blockTimeInCalendar, allowExecutiveRoleChange and allowSubactivities.
      #
      # @see https://uu.unicornuniverse.eu/ues/sesm?&SessFree=ues:UNI-BT:UES.DOC%2FCMD_V1%2FUESQUERY UESQuery Documentation.
      #
      # @return [UU::OS::REST::ResultList<ActivityStateTemplate::ActivityStateTemplateGetActivityStateTemplateList, ActivityStateTemplate::ActivityStateTemplateAttributes>] List of activity templates according to the entered entity. Contains instances of {UU::OS::ActivityTemplate::ActivityTemplateAttributes}.
      #
      # @example
      #   # Returns list of templates of messages whose instances can have subactivities
      #   UU::OS::ActivityTemplate.get_activity_template_list('ues:TERRITORY:META_ARTIFACT',
      #      :query => 'type="MESSAGE" AND allowSubactivities=true')
      def get_activity_template_list(meta_artifact_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(ActivityTemplate)
        dto = ActivityTemplate::ActivityTemplateGetActivityTemplateList.new(criteria)
        svc.add_parameter('pageIndex', dto.page_index)
        svc.add_parameter('pageSize', dto.page_size)
        svc.add_parameter('query', dto.query)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getActivityTemplateList', meta_artifact_uri)
          return UU::OS::REST::ResultList.new(ActivityTemplate::ActivityTemplateGetActivityTemplateList, ActivityTemplate::ActivityTemplateAttributes, res)
        end
      end

    end
  end

end
