require 'uu/os/qos/qos_handler'
require 'uu/os/rest'
require 'uu/os/uesuri'
require 'uu/os/meta_access_role/meta_access_role_add_role_interface'
require 'uu/os/meta_access_role/meta_access_role_attributes'
require 'uu/os/meta_access_role/meta_access_role_copy'
require 'uu/os/meta_access_role/meta_access_role_create'
require 'uu/os/meta_access_role/meta_access_role_export'
require 'uu/os/meta_access_role/meta_access_role_get_export_data'
require 'uu/os/meta_access_role/meta_access_role_get_role_interface_list_item'
require 'uu/os/meta_access_role/meta_access_role_get_role_interface_list'
require 'uu/os/meta_access_role/meta_access_role_remove_role_interface'
require 'uu/os/meta_access_role/meta_access_role_set_attributes'

# Module Unicorn Universe.
module UU
  # Module Operating System.
  module OS
    # Module Meta Role.
    module MetaAccessRole
      extend self

      # Service path
      PATH = 'ues/core/role/UESMetaAccessRole'

      # Returns attributes representing the meta access role specified by the meta_access_role_uri parameter.
      # The command does not change the state of the meta access role in the system.
      #
      # @param [String, UU::OS::UESURI] meta_access_role_uri UESURI of a meta access role whose attributes are returned
      #
      # @return [UU::OS::MetaAccessRole::MetaAccessRoleAttributes] DTO containing the meta access role attributes
      #
      # @example
      #   # Get attributes of a meta access role
      #   UU::OS::MetaAccessRole.get_attributes('ues:TERRITORY:META_ACCESS_ROLE')
      def get_attributes(meta_access_role_uri)
        svc = UU::OS::REST::RemoteClient.new(MetaAccessRole)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getAttributes', meta_access_role_uri)
          return MetaAccessRole::MetaAccessRoleAttributes.new(res)
        end
      end

      # Command for setting attributes of a meta access role. The command cannot modify references to objects like
      # location, competent role and so on.
      #
      # @param [String, UU::OS::UESURI] meta_access_role_uri UESURI of a meta access role whose attributes are returned
      # @param [UU::OS::MetaAccessRole::MetaAccessRoleSetAttributes] meta_access_role DTO containing new attributes of the meta access role
      # @option meta_access_role [String] :name The new name of the meta access role. If nil is set, the name is not modified.
      # @option meta_access_role [String] :code The new code of the meta access role. If nil is set, the code is not modified. The code must be unique in a given territory.
      # @option meta_access_role [String] :description The new description of the meta access role. If nil is set, the description is not modified.
      # @option meta_access_role [String, UU::OS::Artifact::SecurityLevel] :security_level The new security level of the meta access role. If nil is set, the security level is not modified.
      # @option meta_access_role [String, UU::OS::UESURI] :icon_uri UESURI of the new meta access role's icon. If nil is set, the icon is not modified. If nil UESURI ('ues:{[-1]}:{[-1]}:') is set, the meta access role icon is removed and the meta access role uses icon from it's source meta artifact. An example of icon URI: 'ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}ART_077}'.
      # @option meta_access_role [String] :default_name Specifies new default name of the created role according to meta access role. If nil is set, the default name is not modified.
      # @option meta_access_role [String] :default_code Specifies new default code of the created role according to meta access role. If nil is set, the default code is not modified.
      # @option meta_access_role [String, UU::OS::Artifact::SecurityLevel] :default_security_level Specifies new default security level of the created role according to meta access role. If nil is set, the default security is not modified.
      # @option meta_access_role [String, UU::OS::UESURI] :default_icon_uri Specifies new default icon of the created role according to meta access role (specified in metaRoleUri). If nil is set, the default icon is not modified. An example of icon URI: 'ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}ART_077}'.
      # @option meta_access_role [Boolean] :default_searchable_in_territory Specifies if it is possible to search for the role created according to the meta access role in its territory. If nil is set, the attribute is not modified.
      # @option meta_access_role [Boolean] :component_content Defines whether the primarily displayed content of the created Meta access role is component content. Default value is false.           
      # @option meta_access_role [Boolean] :default_component_content Defines whether the primarily displayed content of an access role created according to the meta access role (specified in source_meta_artifact_uri) is component content. When nil is set, the default component content setting is not modified.
      # @return [UU::OS::UESURI] UESURI of the meta access role
      #
      # @example
      #   # Set attributes of a meta access role
      #   UU::OS::MetaAccessRole.set_attributes('ues:TERRITORY:META_ACCESS_ROLE', :name => 'New Meta Access Role', :code => 'NEW_META_ACCESS_ROLE',
      #     :description => 'Meta Access Role designated for testing purposes', :default_searchable_in_territory => true)
      def set_attributes(meta_access_role_uri, meta_access_role = nil)
        svc = UU::OS::REST::RemoteClient.new(MetaAccessRole)
        payload = UU::OS::MetaAccessRole::MetaAccessRoleSetAttributes.new(meta_access_role).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setAttributes', meta_access_role_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Copy a meta access role. The command creates a new meta access role according to specified source
      # meta access role in a specified location. At the very least location meta access role and name must
      # be specified in {MetaAccessRoleCopy} DTO. Competent role for the new meta access role copy will
      # be selected as the most suitable according to specified container and meta access role
      # (executive/authorized role with connected interface), or can be also specified in {MetaAccessRoleCopy} DTO.
      #
      # @param [String, UU::OS::UESURI] source_meta_artifact_uri UESURI of the meta access role which will be copied.
      # @param [UU::OS::MetaAccessRole::MetaAccessRoleCopy] meta_access_role DTO containing attributes of the new meta access role copy.
      # @option meta_access_role [String, UU::OS::UESURI] :location_uri UESURI of the meta model dictionary where to the meta access role will be copied.
      # @option meta_access_role [String] :name Name of the new meta access role copy. The value must not be blank.
      # @option meta_access_role [String] :code Code of the new meta access role copy.
      # @option meta_access_role [String] :description Description of the new meta access role copy.
      # @option meta_access_role [String, UU::OS::UESURI] :competent_role_uri UESURI of the competent role.
      # @option meta_access_role [String, UU::OS::Artifact::SecurityLevel] :security_level Security level.
      # @option meta_access_role [String, UU::OS::UESURI] :icon_uri URI of the new meta access role's icon. Allowed format of icon URI: 'ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}CODE_OF_ICON}'.
      # @option meta_access_role [Boolean] :copy_sheets Specifies whether to copy content from the source meta access role to the new copy. Default value is false.
      # @option meta_access_role [Boolean] :copy_lifecycle Specifies whether to copy workflow from the source meta access role to the new copy. Default value is false.
      # @option meta_access_role [Boolean] :copy_attachments Specifies whether to copy attachments from the source meta access role to the new copy. Default value is false.
      # @option meta_access_role [Boolean] :copy_sheet_templates Specifies whether to copy sheet templates from the source meta access role to the new copy. Default value is false.
      # @option meta_access_role [String] :default_name Specifies default name of the created role according to new meta access role copy.
      # @option meta_access_role [String] :default_code Specifies default code of the created role according to new meta access role copy.
      # @option meta_access_role [String, UU::OS::Artifact::SecurityLevel] :default_security_level Specifies default security level of the created role according to new meta access role copy.
      # @option meta_access_role [String, UU::OS::UESURI] :default_icon_uri Specifies default icon of the created role according to new meta access role copy. Allowed format of icon URI: 'ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}CODE_OF_ICON}'.
      # @option meta_access_role [String] :default_searchable_in_territory Specifies if it is possible to search for the role created according to the meta access role in its territory.
      # @option meta_access_role [Boolean] :component_content Defines whether the primarily displayed content of the created Meta access role is component content. Default value is false.           
      # @option meta_access_role [Boolean] :default_component_content Defines whether the primarily displayed content of an access role created according to the new meta access role is component content. When nil is set, the attribute from the XML (entered in :data option) is used.
      # @return [UU::OS::UESURI] UESURI of the copied meta access role.
      #
      # @example
      #   # Copy a meta access role (including its sheets); change a few attributes
      #   UU::OS::MetaAccessRole.copy('ues:TERRITORY:META_ACCESS_ROLE', :location_uri => 'ues:TERRITORY:APP_PACKAGE', :code => 'META_ACCESS_ROLE_COPY',
      #     :icon_uri => 'ues:#{system}:#{ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{ART_077}',
      #     :security_level => UU::OS::Artifact::SecurityLevel::INTERNAL_USE, :copy_sheets => true)
      def copy(source_meta_artifact_uri, meta_access_role = nil)
        svc = UU::OS::REST::RemoteClient.new(MetaAccessRole)
        svc.timeout = 120 # may take longer than default 60 seconds
        payload = UU::OS::MetaAccessRole::MetaAccessRoleCopy.new(meta_access_role).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('copy', source_meta_artifact_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Creates new role interface connection to the specified meta access role.
      # The role interface has to be in an active state and must not be already added. The role
      # interface can be added to meta access role only provided that role interface is located in the
      # same, child or parent meta model as meta access role.
      #
      # @param [String, UU::OS::UESURI] meta_access_role_uri UESURI of the role where the role interface will be connected
      # @param [UU::OS::MetaAccessRole::MetaAccessRoleAddRoleInterface] role_interface DTO containing role interface that is to be connected
      # @option role_interface [String, UU::OS::UESURI] :role_interface_uri UESURI of the role interface that is to be connected to the specified meta access role.
      #
      # @example
      #   # Connect another role interface to a meta access role
      #   UU::OS::MetaAccessRole.add_role_interface('ues:TERRITORY:META_ACCESS_ROLE', :role_interface_uri => 'ues:TERRITORY:ROLE_INTERFACE')
      def add_role_interface(meta_access_role_uri, role_interface = nil)
        svc = UU::OS::REST::RemoteClient.new(MetaAccessRole)
        payload = UU::OS::MetaAccessRole::MetaAccessRoleAddRoleInterface.new(role_interface).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('addRoleInterface', meta_access_role_uri, payload)
        end
      end

      # Returns list of the role interfaces that are added to the specified meta access role.
      # The list of returned interfaces could be filtered by the name or code. Returned list does
      # not contain any objects filtered out by executed UESQuery. The list is sorted by the name
      # (and codes in case are equal) of an interface by default.
      #
      # See {https://uu.unicornuniverse.eu/ues/sesm?&SessFree=ues:UNI-BT:UES.DOC%2FCMD_V1%2FUESQUERY UESQuery Documentation}.
      #
      # @param [String, UU::OS::UESURI] meta_access_role_uri UESURI of the meta access role for which the added role interfaces are to be listed
      # @param [UU::OS::MetaAccessRole::MetaAccessRoleGetRoleInterfaceList] criteria DTO containing criteria for listing connected role interfaces
      # @option criteria [String] :query Query for filtering and ordering of the result list (more information in {uuQuery - Guideline}[https://plus4u.net/ues/sesm?SessFree=ues%253AUNI-BT%255B210795%255D%253AUES.DOC%252FCMD_V1%252FUESQUERY%255B44191574519623267%255D%253A]). It is possible to filter the result list by the name or code.
      #
      # @return [UU::OS::REST::ResultList<MetaAccessRole::MetaAccessRoleGetRoleInterfaceList, MetaAccessRole::MetaAccessRoleGetRoleInterfaceListItem>] List of DTO representing role interfaces which are connected to the specified meta access role
      #
      # @example
      #   # Get a list of all role interfaces which are connected to the specified meta access role
      #   UU::OS::MetaAccessRole.get_role_interface_list('ues:TERRITORY:META_ACCESS_ROLE')
      #
      #   # Get a list of role interfaces with name that starts with string John.
      #   UU::OS::MetaAccessRole.get_role_interface_list('ues:TERRITORY:META_ACCESS_ROLE', :query => "name='John%'")
      def get_role_interface_list(meta_access_role_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(MetaAccessRole)
        dto = MetaAccessRole::MetaAccessRoleGetRoleInterfaceList.new(criteria)
        svc.add_parameter('pageIndex', dto.page_index)
        svc.add_parameter('pageSize', dto.page_size)
        svc.add_parameter('query', dto.query)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getRoleInterfaceList', meta_access_role_uri)
          return UU::OS::REST::ResultList.new(MetaAccessRole::MetaAccessRoleGetRoleInterfaceList, MetaAccessRole::MetaAccessRoleGetRoleInterfaceListItem, res)
        end
      end

      # Removes role interface connection from the specified meta access role.
      # Only added interfaces can be removed from meta access role.
      #
      # @param [String, UU::OS::UESURI] meta_access_role_uri UESURI of the meta access role where the role interface will be disconnected.
      # @param [UU::OS::MetaAccessRole::MetaAccessRoleRemoveRoleInterface] role_interface DTO containing role interface that to be disconnected
      # @option role_interface [String, UU::OS::UESURI] :role_interface_uri UESURI of the role interface that to be removed from the specified meta access role.
      #
      # @example
      #   # Remove a connected role interface from a meta access role
      #   UU::OS::MetaAccessRole.remove_role_interface('ues:TERRITORY:META_ACCESS_ROLE', :role_interface_uri => 'ues:TERRITORY:ROLE_INTERFACE')
      def remove_role_interface(meta_access_role_uri, role_interface = nil)
        svc = UU::OS::REST::RemoteClient.new(MetaAccessRole)
        payload = UU::OS::MetaAccessRole::MetaAccessRoleRemoveRoleInterface.new(role_interface).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('removeRoleInterface', meta_access_role_uri, payload)
        end
      end

      # Exports a meta access role to the XML file, which is saved to the export storage. The export is an asynchronous process.<br>
      # It is possible to wait for the end of the process. More information in {UU::OS::Env::Process Process} and in {UU::OS::REST::Future Future}.<br>
      # To get exported data from the storage use {#get_export_data get_export_data}.
      #
      # @param [String, UU::OS::UESURI] meta_access_role_uri UESURI of the meta access role to export.
      #
      # @return [UU::OS::UESURI] UESURI of the export process.
      #
      # @example
      #   # Start exporting a meta access role.
      #   process_uri = UU::OS::MetaAccessRole.export('ues:TERRITORY:META_ACCESS_ROLE')
      #
      #   # Wait for the end of the process.
      #   export_uri = UU::OS::REST::Future.new(process_uri).get(nil, 600)
      #
      #   # Get exported XML file.
      #   exported_data = UU::OS::MetaAccessRole.get_export_data(export_uri)
      def export(meta_access_role_uri)
        svc = UU::OS::REST::RemoteClient.new(MetaAccessRole)
        payload = UU::OS::MetaAccessRole::MetaAccessRoleExport.new(nil).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('export', meta_access_role_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Returns a XML file generated by the export command.
      #
      # @param [String, UU::OS::UESURI] meta_access_role_uri UESURI of the meta artifact we want to get the exported XML file from.
      # @param [UU::OS::MetaAccessRole::MetaAccessRoleGetExportData] meta_access_role Specific options.
      # @option meta_access_role [String] :requested_mime_type MIME type of the returned value. Possible values are 'application/xml' for XML and 'application/zip' for ZIP. ZIP is returned by default.
      #
      # @return [UU::OS::REST::BinaryValue] Data file of an exported meta artifact (XML or ZIP containing XML).
      #
      # @example
      #   # Start exporting a meta access role.
      #   process_uri = UU::OS::MetaAccessRole.export('ues:TERRITORY:META_ACCESS_ROLE')
      #
      #   # Wait for the end of the process.
      #   export_uri = UU::OS::REST::Future.new(process_uri).get(nil, 600)
      #
      #   # Get exported XML file.
      #   exported_data = UU::OS::MetaAccessRole.get_export_data(export_uri)
      def get_export_data(meta_access_role_uri, meta_access_role = nil)
        svc = UU::OS::REST::RemoteClient.new(MetaAccessRole)
        dto = UU::OS::MetaAccessRole::MetaAccessRoleGetExportData.new(meta_access_role)
        if dto.requested_mime_type
          svc.add_parameter('requestedMimeType', dto.requested_mime_type)
        end
        UU::OS::QoS::QoSHandler.auto_retry do
          return svc.get_binary('getExportData', meta_access_role_uri)
        end
      end

      # Creates a meta access role from an exported XML file in the specified location. Source meta access role must be derived (at least indirectly) from a meta access role which is a part of UU.OS uuApp. Creation of meta access role is divided into synchronous and asynchronous part. During the synchronous part, the meta access role is created. During the asynchronous part, the template objects are created. If the asynchronous process fails and the Problem state is set on meta access role, it is possible to solve the problem (e.g. add reference_map) and run the command again.
      # @deprecated The uuCommand MetaAccessRole.create is used for creating meta access role from the XML file temporarily. This feature will be removed in a future version and replaced by the specific uuCommands for creating template objects.
      #
      # @param [String, UU::OS::UESURI] location_uri UESURI of the meta model dictionary where new meta access role should be created.
      # @param [UU::OS::MetaAccessRole::MetaAccessRoleCreate] meta_access_role Specific options.
      # @option meta_access_role [String, File, UU::OS::REST::BinaryValue] :data XML or zip compressed XML containing informations about meta access role (e.g. exported with {export})
      # @option meta_access_role [String, UU::OS::UESURI] :competent_role_uri UESURI of the role that will be competent for the new meta access role. If nil is set, the attribute from the XML (entered in :data option) is used.
      # @option meta_access_role [String] :name Name of the new meta access role. If nil is set, the attribute from the XML (entered in :data option) is used.
      # @option meta_access_role [String] :code Code of the new meta access role. If nil is set, the attribute from the XML (entered in :data option) is used.
      # @option meta_access_role [String] :description Description of the new meta access role. If nil is set, the attribute from the XML (entered in :data option) is used.
      # @option meta_access_role [String, UU::OS::Artifact::SecurityLevel] :security_level Security level of the new meta access role. If nil is set, the attribute from the XML (entered in :data option) is used.
      # @option meta_access_role [String, UU::OS::UESURI] :icon_uri UESURI of the new meta access role icon. If nil is set, the attribute from the XML (entered in :data option) is used. An example of icon URI: 'ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}ART_077}'.
      # @option meta_access_role [String] :default_name Default name of a access role that will be created according to the new meta access role. If nil is set, the attribute from the XML (entered in :data option) is used.
      # @option meta_access_role [String] :default_code Default code of a access role that will be created according to the new meta access role. If nil is set, the attribute from the XML (entered in :data option) is used.
      # @option meta_access_role [String, UU::OS::UESURI] :default_icon_uri UESURI of the access role icon that will be created according to the new meta access role. If nil is set, the attribute from the XML (entered in :data option) is used. An example of icon URI: 'ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}ART_077}'.
      # @option meta_access_role [String, UU::OS::Artifact::SecurityLevel] :default_security_level Default security level of the new meta access role. If nil is set, the attribute from the XML (entered in :data option) is used.
      # @option meta_access_role [Boolean] :default_searchable_in_territory Flag indicating whether a role that will be created according to the new meta access role is searchable in the territory.
      # @option meta_access_role [Boolean] :import_life_cycle_template Flag indicating whether the life cycle template will be created according to specification in the entered XML. Default value is true.
      # @option meta_access_role [Boolean] :import_property_templates Flag indicating whether the property templates will be created according to specification in the entered XML. Default value is true.
      # @option meta_access_role [Boolean] :import_sheet_templates Flag indicating whether the sheet templates will be created according to specification in the entered XML. Default value is true.
      # @option meta_access_role [Boolean] :import_properties Flag indicating whether the properties will be created according to specification in the entered XML. Default value is true.
      # @option meta_access_role [Boolean] :import_sheets Flag indicating whether the sheets will be created according to specification in the entered XML. Default value is true.
      # @option meta_access_role [Boolean] :import_attachments Flag indicating whether the attachments will be created according to specification in the entered XML. Default value is true.      
      # @option meta_access_role [Boolean] :import_meta_permissions Flag indicating whether the meta permissions will be created according to specification in the entered XML. Default value is true.
      # @option meta_access_role [Boolean] :import_permissions Flag indicating whether the permissions will be created according to specification in the entered XML. Default value is true.
      # @option meta_access_role [Boolean] :import_use_cases Flag indicating whether the use cases will be created according to specification in the entered XML. Default value is true.
      # @option meta_access_role [Boolean] :import_masks Flag indicating whether the permission masks will be created according to specification in the entered XML. Default value is true.
      # @option meta_access_role [String] :reference_map JSON describing the linkage of references specified in XML to another objects (e.g. references in properties, activity related artifacts. ...).
      # @option meta_access_role [Boolean] :default_component_content Defines whether the primarily displayed content of an access role created according to the new meta access role is component content. When nil is set, the attribute from the XML (entered in :data option) is used.
      # @return [UU::OS::UESURI] UESURI of the new meta access role.
      #
      # @example
      #   # Get exported data from an existing meta access role.
      #   exported_data = UU::OS::MetaAccessRole.get_export_data('ues:TERRITORY:META_ACCESS_ROLE', :requested_mime_type=>'application/xml')
      #
      #   # Create a new meta access role according to exported one without creating sheets and attachments specified in the XML. Also set different code and name.
      #   UU::OS::MetaAccessRole.create('ues:TERRITORY:APP_PACKAGE', :code => 'NEW_META_ACCESS_ROLE', :name => 'New Meta Access Role', :data => exported_data.data, :import_sheets => false, :import_attachments => false)
      #
      #   # Create a new meta access role and set new objects to the references.
      #   UU::OS::MetaAccessRole.create('ues:TERRITORY:APP_PACKAGE', :data => exported_data.data, :reference_map => '[{"ues:OLD_TERRITORY:OLD_ARTIFACT1":"ues:TERRITORY:ARTIFACT1"},{"ues:OLD_TERRITORY:OLD_ARTIFACT2:OLD_OBJECT":"ues:TERRITORY:ARTIFACT2:OBJECT"}]')
      def create(location_uri, meta_access_role = nil)
        svc = UU::OS::REST::RemoteClient.new(MetaAccessRole)
        dto = UU::OS::MetaAccessRole::MetaAccessRoleCreate.new(meta_access_role)
        payload = dto.to_hash
        payload.delete(:data)
        tmpfile = Tempfile.new("MetaAccessRoleCreate")
        begin
          if (!dto.data.data.respond_to?(:read))
            tmpfile.write(dto.data.data)
            payload[:data] = tmpfile
          elsif (dto.data.data.respond_to?(:read) && !dto.data.data.respond_to?(:path))
            tmpfile.binmode
            dto.data.data.rewind
            tmpfile.write(dto.data.data.read)
            payload[:data] = tmpfile
          else
            payload[:data] = dto.data.to_hash(false)
          end
          tmpfile.flush
          tmpfile.rewind
              
          UU::OS::QoS::QoSHandler.auto_retry do
            res = svc.post('create', location_uri, payload)
            return UU::OS::UESURI.new(res)
          end
        ensure
          tmpfile.close
          tmpfile.unlink
        end
      end

    end
  end

end
