require 'uu/os/qos/qos_handler'
require 'uu/os/rest'
require 'uu/os/uesuri'
require 'uu/os/role_interface/role_interface_create'
require 'uu/os/role_interface/role_interface_attributes'
require 'uu/os/role_interface/role_interface_set_attributes'
require 'uu/os/role_interface/role_interface_add_meta_artifact'
require 'uu/os/role_interface/role_interface_remove_meta_artifact'
require 'uu/os/role_interface/role_interface_get_meta_artifact_list'
require 'uu/os/role_interface/role_interface_get_meta_artifact_list_item'
require 'uu/os/role_interface/role_interface_export'
require 'uu/os/role_interface/role_interface_get_export_data'
require 'uu/os/role_interface/role_interface_get_role_list'
require 'uu/os/role_interface/role_interface_get_role_list_item'

# Module Unicorn Universe.
module UU
  # Module Operating System.
  module OS
    # Module Role Interface.
    module RoleInterface
      extend self

      # Service path
      PATH = 'ues/core/role/UESRoleInterface'

      # Creates a new role interface. The command creates a new role interface in a specified
      # meta model dictionary. At the very least meta artifact, name and code must be specified
      # in {RoleInterfaceCreate} Object. Competent role for the new group will
      # be selected as the most suitable according to specified container and meta artifact (executive/authorized
      # role with connected interface), or can be also specified in {RoleInterfaceCreate}
      # Object.
      #
      # @param [String, UU::OS::UESURI] location_uri UESURI of the meta model dictionary where the role interface will be created.
      # @param [RoleInterfaceCreate] role_interface Object containing attributes of the new role interface.
      # @option role_interface [String, UU::OS::UESURI] :meta_artifact_uri UESURI of the meta artifact. Only a meta artifact with the template class of role interface can be used. If the rights does not permit to create a new role interface from the meta artifact in the specified location, the create command fails.
      # @option role_interface [String] :name Name of the new role interface. If nil is set, template name from meta artifact is used if set, otherwise the create command fails.
      # @option role_interface [String] :code Code of the new role interface. If nil is set, the code is generated automatically using the system sequence. The code must be unique in the given territory, otherwise the create command fails.
      # @option role_interface [String] :description Description of the new role interface. If nil is set, from meta artifact is used if set, otherwise no description is set on the new role interface.
      # @option role_interface [String, UU::OS::UESURI] :competent_role_uri UESURI of the competent role. If nil is set, the executive role from authorization is used. If no role is authorized to create the new role interface into specified container, the create command fails. If the selected role doesn't have connected the role interface with the selected meta artifact, the create command fails.
      # @option role_interface [String, UU::OS::Artifact::SecurityLevel] :security_level Security level. If nil is set, security level is taken from the meta artifact template security level. If the meta artifact template security level is higher than the user's security clearance, the create command fails.
      # @option role_interface [String, UU::OS::UESURI] :icon_uri URI of the role interface's icon. If nil is set, the meta model uses icon from meta artifact. An example of icon URI: 'ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}ART_077}'.
      # @option role_interface [Boolean] :component_content Defines whether the primarily displayed content of the created AppMetaModel is component content. Default value is false.
      #
      # @return [UU::OS::UESURI] UESURI of the new role interface
      #
      # @example
      #   # Create a new role interface.
      #   UU::OS::RoleInterface.create('ues:TERRITORY:ROLE_INTERFACE', :code => 'NEW_ROLE_INTERFACE', :name => 'New Role Interface')
      def create(location_uri, role_interface = nil)
        svc = UU::OS::REST::RemoteClient.new(RoleInterface)
        payload = UU::OS::RoleInterface::RoleInterfaceCreate.new(role_interface).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('create', location_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Returns attributes representing the role interface specified by
      # the role_interface_uri parameter. The command does not change the state of the role
      # interface in the system.
      #
      # @param [String, UU::OS::UESURI] role_interface_uri UESURI of a role interface whose attributes are returned
      #
      # @return [RoleInterfaceAttributes] Object containing the role interface attributes
      #
      # @example
      #   # Get attributes of a role interface
      #   UU::OS::RoleInterface.get_attributes('ues:TERRITORY:ROLE_INTERFACE')
      def get_attributes(role_interface_uri)
        svc = UU::OS::REST::RemoteClient.new(RoleInterface)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getAttributes', role_interface_uri)
          return RoleInterface::RoleInterfaceAttributes.new(res)
        end
      end

      # Command for setting attributes of a role interface. The command cannot modify references
      # to objects like location, competent role and so on.
      #
      # @param [String, UU::OS::UESURI] role_interface_uri UESURI of the role interface whose attributes are to be changed
      # @param [RoleInterfaceSetAttributes] role_interface Object containing new attributes the role interface
      # @option role_interface [String] :name The new name of the role interface. If nil is set, the name is not modified.
      # @option role_interface [String] :code The new code of the role interface. If nil is set, the code is not modified. The code must be unique in a given territory, otherwise the set_attributes command fails.
      # @option role_interface [String] :description The new description of the role interface. If nil is set, the description is not modified.
      # @option role_interface [String, UU::OS::Artifact::SecurityLevel] :security_level The new security level of the role interface. If nil is set, the security level is not modified. If the role interface security level is higher than the user's security clearance, the set_attributes command fails.
      # @option role_interface [String, UU::OS::UESURI] :icon_uri URI of the role interface's icon. If nil is set, the icon is not modified. If nil UESURI ("ues:{[-1]}:{[-1]}:") is set, the role interface's icon is removed and the role interface uses icon from meta artifact. An example of icon URI: 'ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}ART_077}'.
      # @option role_interface [Boolean] :component_content Defines whether the primarily displayed content of the created AppMetaModel is component content. Default value is false.
      #
      # @return [UU::OS::UESURI] UESURI of the role interface
      #
      # @example
      #   # Change most of attributes of a role interface
      #   UU::OS::RoleInterface.set_attributes('ues:TERRITORY:ROLE_INTERFACE',
      #         :name => 'New Role Interface Name',
      #         :code => 'NEW_ROLE_INTERFACE_CODE',
      #         :description => 'Some sample description',
      #         :security_level => UU::OS::Artifact::SecurityLevel::INTERNAL_USE
      #   )
      def set_attributes(role_interface_uri, role_interface = nil)
        svc = UU::OS::REST::RemoteClient.new(RoleInterface)
        payload = UU::OS::RoleInterface::RoleInterfaceSetAttributes.new(role_interface).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setAttributes', role_interface_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Deletes the specified role interface from the system. The role interface
      # is deleted even if it contains a locked sheet or attachment. The role interface
      # cannot be deleted if:
      # the role interface's workflow is not created,
      # a property contains a reference to this role interface,
      # a time sheet exists with a reference to this role interface.
      #
      # @param [String, UU::OS::UESURI] role_interface_uri UESURI of the role interface which is about to be deleted.
      #
      # @example
      #   # Delete a role interface
      #   UU::OS::RoleInterface.delete('ues:TERRITORY:ROLE_INTERFACE')
      def delete(role_interface_uri)
        svc = UU::OS::REST::RemoteClient.new(RoleInterface)
        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('delete', role_interface_uri)
        end
      end

      # Creates new meta artifact connection to the specified role interface.
      #
      # @param [String, UU::OS::UESURI] role_interface_uri UESURI of the role interface where the meta artifacts will be connected
      # @param [RoleInterfaceAddMetaArtifact] role_interface Object containing meta artifact that is to be connected
      # @option role_interface [String, UU::OS::UESURI] :meta_artifact_uri UESURI of the meta artifact that is to be connected to the specified role interface.
      #
      # @example
      #   # Connect a meta artifact to a role interface
      #   UU::OS::RoleInterface.add_meta_artifact('ues:TERRITORY:ROLE_INTERFACE',
      #         :meta_artifact_uri => 'ues:TERRITORY:META_ARTIFACT'
      #   )
      def add_meta_artifact(role_interface_uri, role_interface = nil)
        svc = UU::OS::REST::RemoteClient.new(RoleInterface)
        payload = UU::OS::RoleInterface::RoleInterfaceAddMetaArtifact.new(role_interface).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('addMetaArtifact', role_interface_uri, payload)
        end
      end

      # Removes meta artifact connection from the specified role interface.
      # Only connected meta artifacts can be disconnected from the role interface.
      #
      # @param [String, UU::OS::UESURI] role_interface_uri UESURI of the role interface where the meta artifacts will be disconnected.
      # @param [RoleInterfaceRemoveMetaArtifact] role_interface Object containing meta artifact that to be disconnected
      # @option role_interface [String, UU::OS::UESURI] :meta_artifact_uri UESURI of the meta artifact that is to be disconnected from the specified role interface. If the meta artifact is not connected, the disconnect command fails.
      #
      # @example
      #   # Disconnect a meta artifact from a role interface
      #   UU::OS::RoleInterface.remove_meta_artifact('ues:TERRITORY:ROLE_INTERFACE',
      #         :meta_artifact_uri => 'ues:TERRITORY:META_ARTIFACT'
      #   )
      def remove_meta_artifact(role_interface_uri, role_interface = nil)
        svc = UU::OS::REST::RemoteClient.new(RoleInterface)
        payload = UU::OS::RoleInterface::RoleInterfaceRemoveMetaArtifact.new(role_interface).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('removeMetaArtifact', role_interface_uri, payload)
        end
      end

      # Returns list of the meta artifacts that are connected to the specified role interface.
      # Returned list does not contain any objects filtered out by executed UESQuery.
      # The list of returned meta artifacts could be filtered by the name or code. The list
      # is sorted by the name (and codes in case are equal). See the UES query documentation
      # for more information.
      #
      # @see https://uu.unicornuniverse.eu/ues/sesm?&SessFree=ues:UNI-BT:UES.DOC%2FCMD_V1%2FUESQUERY UESQuery Documentation.
      #
      # @param [String, UU::OS::UESURI] role_interface_uri UESURI of the role interface for which the connected meta artifacts are to be listed
      # @param [RoleInterfaceGetMetaArtifactList] criteria Object containing criteria for listing connected meta artifacts. Available parameters are: {RoleInterfaceGetMetaArtifactListItem#name name} and {RoleInterfaceGetMetaArtifactListItem#code code}
      # @option criteria [String] :query The <b>UES query</b> string. It has to conform the UESQuery syntax (see documentation for more info). Filtering and order of the result list can be specified there. If nil is set, no filtering is applied and entries are ordered by name of casted subjects (and code as the second criteria).
      #
      # @return [UU::OS::REST::ResultList<RoleInterface::RoleInterfaceGetMetaArtifactList, RoleInterface::RoleInterfaceGetMetaArtifactListItem>] List of Objects representing meta artifacts which are connected to the specified role interface
      #
      # @example
      #   # List all connected meta artifacts
      #   UU::OS::RoleInterface.get_meta_artifact_list('ues:TERRITORY:ROLE_INTERFACE')
      #
      #   # List connected meta artifacts whose codes start with "META" prefix
      #   UU::OS::RoleInterface.get_meta_artifact_list('ues:TERRITORY:ROLE_INTERFACE',
      #         :query => 'code="META%"'
      #   )
      def get_meta_artifact_list(role_interface_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(RoleInterface)
        dto = RoleInterface::RoleInterfaceGetMetaArtifactList.new(criteria)
        svc.add_parameter('pageIndex', dto.page_index)
        svc.add_parameter('pageSize', dto.page_size)
        svc.add_parameter('query', dto.query)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getMetaArtifactList', role_interface_uri)
          return UU::OS::REST::ResultList.new(RoleInterface::RoleInterfaceGetMetaArtifactList, RoleInterface::RoleInterfaceGetMetaArtifactListItem, res)
        end
      end

      # Exports a role interface to the XML file, which is saved to the export storage. The export is an asynchronous process.<br>
      # It is possible to wait for the end of the process. More information in {UU::OS::Env::Process Process} and in {UU::OS::REST::Future Future}.
      #
      # @param [String, UU::OS::UESURI] role_interface_uri UESURI of the role interface to export.
      #
      # @return [UU::OS::UESURI] UESURI of the export process.
      #
      # @example
      #   # Start exporting a role interface.
      #   process_uri = UU::OS::RoleInterface.export('ues:TERRITORY:ROLE_INTERFACE')
      #
      #   # Wait for the end of the process.
      #   export_uri = UU::OS::REST::Future.new(process_uri).get(:timeout=>600)
      def export(role_interface_uri)
        svc = UU::OS::REST::RemoteClient.new(RoleInterface)
        payload = UU::OS::RoleInterface::RoleInterfaceExport.new(nil).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('export', role_interface_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Returns a XML file generated by the {#export export} command.
      #
      # @param [String, UU::OS::UESURI] role_interface_uri UESURI of the role interface we want to get the exported XML file from.
      # @param [UU::OS::RoleInterface::RoleInterfaceGetExportData] role_interface Specific options.
      # @option role_interface [String, UU::OS::Artifact::ExportMimeType] :requested_mime_type MIME type of the returned value. ZIP is returned by default.
      #
      # @return [UU::OS::REST::BinaryValue] Data file of an exported role interface (XML or ZIP containing XML).
      #
      # @example
      #   # Start exporting a role interface
      #   process_uri = UU::OS::RoleInterface.export('ues:TERRITORY:ROLE_INTERFACE')
      #
      #   # Wait for the end of the process.
      #   export_uri = UU::OS::REST::Future.new(process_uri).get(nil, 600)
      #
      #   # Get exported XML file.
      #   exported_data = UU::OS::RoleInterface.get_export_data(export_uri, :requested_mime_type => 'application/xml')
      def get_export_data(role_interface_uri, role_interface = nil)
        svc = UU::OS::REST::RemoteClient.new(RoleInterface)
        dto = UU::OS::RoleInterface::RoleInterfaceGetExportData.new(role_interface)
        if dto.requested_mime_type
          svc.add_parameter('requestedMimeType', dto.requested_mime_type)
        end
        UU::OS::QoS::QoSHandler.auto_retry do
          return svc.get_binary('getExportData', role_interface_uri)
        end
      end

      # Returns list of roles, which the specified role interface is connected to.
      #
      # @param [String, UU::OS::UESURI] role_interface_uri URI of the role interface to retrieve the roles it is connected to.
      # @param [RoleInterfaceGetRoleList] criteria Object containing paging criteria.
      #
      # @return [UU::OS::REST::ResultList<RoleInterface::RoleInterfaceGetRoleList, RoleInterface::RoleInterfaceGetRoleListItem>] Returns list of Objects representing roles, which the specified role interface is connected to. Contains instances of (RoleInterface::RoleInterfaceGetRoleListItem).
      #
      # @example
      #   # Retrieve all roles, which an interface is connected to
      #   roles = UU::OS::RoleInterface.get_role_list('ues:TERRITORY:ROLE_INTERFACE')
      def get_role_list(role_interface_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(RoleInterface)
        dto = RoleInterfaceGetRoleList.new(criteria)
        svc.add_parameter('pageIndex', dto.page_index)
        svc.add_parameter('pageSize', dto.page_size)

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getRoleList', role_interface_uri)
          return UU::OS::REST::ResultList.new(RoleInterface::RoleInterfaceGetRoleList, RoleInterface::RoleInterfaceGetRoleListItem, res)
        end
      end

    end
  end

end
